// ライセンス: GPL2

#define _DEBUG
#include "jddebug.h"

#include "fontcolorpref.h"
#include "colorid.h"
#include "viewfactory.h"
#include "config/globalconf.h"
#include "dbtree/interface.h"
#include "jdlib/miscutil.h"
#include "jdlib/misctime.h"

#include <sstream>
#include <sys/time.h>


using namespace CORE;

FontColorPref::FontColorPref( const std::string& url )
: SKELETON::PrefDiag( url ),
m_preview( 0 )
{
    // フォント設定をセット
    set_font_settings( "スレッド", CONFIG::get_fontname_main() );
    set_font_settings( "スレッド一覧", CONFIG::get_fontname_tree() );
    set_font_settings( "板一覧", CONFIG::get_fontname_tree_board() );
    set_font_settings( "ポップアップ", CONFIG::get_fontname_popup() );
    set_font_settings( "書き込みウィンドウ", CONFIG::get_fontname_message() );

    strict_char_width = CONFIG::get_strict_char_width();

    // 色設定をセット
    set_color_settings( "文字: スレッド", CONFIG::get_color_char() );
    set_color_settings( "文字: 名前欄", CONFIG::get_color_char_name() );
    set_color_settings( "文字: 名前欄(トリップ等)", CONFIG::get_color_char_name_b() );
    set_color_settings( "文字: メール欄(非sage)", CONFIG::get_color_char_age() );
    set_color_settings( "文字: 選択範囲", CONFIG::get_color_char_selection() );
    set_color_settings( "文字: ハイライト", CONFIG::get_color_char_highlight() );
    set_color_settings( "文字: ブックマーク", CONFIG::get_color_char_bookmark() );
    set_color_settings( "文字: リンク(通常)", CONFIG::get_color_char_link() );
    set_color_settings( "文字: リンク(複数)", CONFIG::get_color_char_link_pur() );
    set_color_settings( "文字: リンク(多数)", CONFIG::get_color_char_link_red() );
    set_color_settings( "文字: 画像(キャッシュ無)", CONFIG::get_color_char_img_nocache() );
    set_color_settings( "文字: 画像(キャッシュ有)", CONFIG::get_color_char_img_cached() );
    set_color_settings( "文字: 画像(ロード中)", CONFIG::get_color_char_img_loading() );
    set_color_settings( "文字: 画像(エラー)", CONFIG::get_color_char_img_err() );
    set_color_settings( "背景: スレッド", CONFIG::get_color_back() );
    set_color_settings( "背景: ツリービュー", CONFIG::get_color_back_tree() );
    set_color_settings( "背景: 板ツリー", CONFIG::get_color_back_tree_board() );
    set_color_settings( "背景: ポップアップ", CONFIG::get_color_back_popup() );
    set_color_settings( "背景: 選択範囲", CONFIG::get_color_back_selection() );
    set_color_settings( "背景: ハイライト", CONFIG::get_color_back_highlight() );
    set_color_settings( "特殊: 新着セパレータ", CONFIG::get_color_separator_new() );

    pack_widget();
}


FontColorPref::~FontColorPref()
{
    if( m_preview ) delete m_preview;

}


//
// 各ウィジェットを追加
//
void FontColorPref::pack_widget()
{
    // フォント
    m_combo_font.set_active( 0 );
    m_fontbutton.set_title( font_names[ 0 ] );
    m_fontbutton.set_font_name( font_values[ 0 ] );

    m_hbox_font.set_spacing( 8 );
    m_hbox_font.set_border_width( 8 );
    m_hbox_font.pack_start( m_combo_font, Gtk::PACK_SHRINK );
    m_hbox_font.pack_start( m_fontbutton, Gtk::PACK_EXPAND_WIDGET, 8 );

    m_vbox_font.pack_start( m_hbox_font, Gtk::PACK_SHRINK );

    m_checkbutton_font.set_label( "フォント幅の近似計算を厳密に行う");
    m_checkbutton_font.set_active( strict_char_width );

    m_vbox_font.pack_start( m_checkbutton_font, Gtk::PACK_SHRINK );

    m_frame_font.set_label( "フォントの設定" );
    m_frame_font.add( m_vbox_font );

    get_vbox()->pack_start( m_frame_font, Gtk::PACK_SHRINK );

    m_combo_font.signal_changed().connect( sigc::mem_fun( *this, &FontColorPref::slot_combo_font_changed ) );
    m_fontbutton.signal_font_set().connect( sigc::mem_fun( *this, &FontColorPref::slot_fontbutton_on_set ) );
    m_checkbutton_font.signal_toggled().connect( sigc::mem_fun( *this, &FontColorPref::slot_checkbutton_font_toggled ) );

    m_combo_color.set_active( 0 );
    m_colorbutton.set_title( color_names[ 0 ] );
    m_colorbutton.set_color( color_values[ 0 ] );

    m_hbox_color.set_spacing( 8 );
    m_hbox_color.set_border_width( 8 );
    m_hbox_color.pack_start( m_combo_color, Gtk::PACK_SHRINK );

    m_hbox_color.pack_start( m_colorbutton, Gtk::JUSTIFY_RIGHT, Gtk::PACK_SHRINK );

    m_vbox_color.pack_start( m_hbox_color, Gtk::PACK_EXPAND_WIDGET, 8 );

    m_frame_color.set_label( "色の設定" );
    m_frame_color.add( m_vbox_color );

    get_vbox()->pack_start( m_frame_color, Gtk::PACK_SHRINK );

    m_combo_color.signal_changed().connect( sigc::mem_fun( *this, &FontColorPref::slot_combo_color_changed ) );
    m_colorbutton.signal_color_set().connect( sigc::mem_fun( *this, &FontColorPref::slot_colorbutton_on_set ) );


    // プレビュー
    m_preview = CORE::ViewFactory( CORE::VIEW_ARTICLEPREVIEW, get_url() );

    m_frame_preview.set_border_width( 8 );
    m_frame_preview.add( *m_preview );

    preview_create();

    get_vbox()->pack_start( m_frame_preview, Gtk::PACK_EXPAND_WIDGET );


    // 全体
    get_vbox()->set_spacing( 0 );
    set_title( "フォントと色の設定" );
    set_border_width( 8 );
    show_all_children();
    resize( 480, 360 );
}


//
// 設定ダイアログで OK が押された
//
void FontColorPref::slot_ok_clicked()
{
    CONFIG::set_fontname_main( font_values[ FONTNAME_MAIN ] );
    CONFIG::set_fontname_tree( font_values[ FONTNAME_TREE ] );
    CONFIG::set_fontname_tree_board( font_values[ FONTNAME_TREE_BOARD ] );
    CONFIG::set_fontname_popup( font_values[ FONTNAME_POPUP ] );
    CONFIG::set_fontname_message( font_values[ FONTNAME_MESSAGE ] );

    CONFIG::set_strict_char_width( strict_char_width );

    CONFIG::set_color_char( MISC::color_to_str( color_values[ COLOR_CHAR ] ) );
    CONFIG::set_color_char_name( MISC::color_to_str( color_values[ COLOR_CHAR_NAME ] ) );
    CONFIG::set_color_char_name_b( MISC::color_to_str( color_values[ COLOR_CHAR_NAME_B ] ) );
    CONFIG::set_color_char_age( MISC::color_to_str( color_values[ COLOR_CHAR_AGE ] ) );
    CONFIG::set_color_char_selection( MISC::color_to_str( color_values[ COLOR_CHAR_SELECTION ] ) );
    CONFIG::set_color_char_highlight( MISC::color_to_str( color_values[ COLOR_CHAR_HIGHLIGHT ] ) );
    CONFIG::set_color_char_bookmark( MISC::color_to_str( color_values[ COLOR_CHAR_BOOKMARK ] ) );
    CONFIG::set_color_char_link( MISC::color_to_str( color_values[ COLOR_CHAR_LINK ] ) );
    CONFIG::set_color_char_link_pur( MISC::color_to_str( color_values[ COLOR_CHAR_LINK_PUR ] ) );
    CONFIG::set_color_char_link_red( MISC::color_to_str( color_values[ COLOR_CHAR_LINK_RED ] ) );
    CONFIG::set_color_char_img_nocache( MISC::color_to_str( color_values[ COLOR_CHAR_IMG_NOCACHE ] ) );
    CONFIG::set_color_char_img_cached( MISC::color_to_str( color_values[ COLOR_CHAR_IMG_CACHED ] ) );
    CONFIG::set_color_char_img_loading( MISC::color_to_str( color_values[ COLOR_CHAR_IMG_LOADING ] ) );
    CONFIG::set_color_char_img_err( MISC::color_to_str( color_values[ COLOR_CHAR_IMG_ERR ] ) );
    CONFIG::set_color_back( MISC::color_to_str( color_values[ COLOR_BACK ] ) );
    CONFIG::set_color_back_tree( MISC::color_to_str( color_values[ COLOR_BACK_TREE ] ) );
    CONFIG::set_color_back_tree_board( MISC::color_to_str( color_values[ COLOR_BACK_TREE_BOARD ] ) );
    CONFIG::set_color_back_popup( MISC::color_to_str( color_values[ COLOR_BACK_POPUP ] ) );
    CONFIG::set_color_back_selection( MISC::color_to_str( color_values[ COLOR_BACK_SELECTION ] ) );
    CONFIG::set_color_back_highlight( MISC::color_to_str( color_values[ COLOR_BACK_HIGHLIGHT ] ) );
    CONFIG::set_color_separator_new( MISC::color_to_str( color_values[ COLOR_SEPARATOR_NEW ] ) );
}


//
// フォント設定の名前と設定値をセット
//
void FontColorPref::set_font_settings( const std::string& name, const std::string& value )
{
    if( ! name.empty() && ! value.empty() )
    {
        font_names.push_back( name );
        m_combo_font.append_text( name );

        font_values.push_back( value );
    }
}


//
// 色設定の名前と設定値をセット
//
void FontColorPref::set_color_settings( const std::string& name, const std::string& value )
{
    if( ! name.empty() )
    {
        color_names.push_back( name );
        m_combo_color.append_text( name );

        color_values.push_back( MISC::str_to_color( value ) );
    }
}


//
// フォント設定のコンボボックスの状態が変わった
//
void FontColorPref::slot_combo_font_changed()
{
    int num = m_combo_font.get_active_row_number();

    m_fontbutton.set_title( font_names[ num ] );

    m_fontbutton.set_font_name( font_values[ num ] );
}


//
// 色設定のコンボボックスの状態が変わった
//
void FontColorPref::slot_combo_color_changed()
{
    int num = m_combo_color.get_active_row_number();

    m_colorbutton.set_title( color_names[ num ] );

    m_colorbutton.set_color( color_values[ num ] );
}


//
// チェックボックスの状態が変わった
//
void FontColorPref::slot_checkbutton_font_toggled()
{
    strict_char_width = m_checkbutton_font.property_active();
}


//
// フォント選択ダイアログで OK が押された
//
void FontColorPref::slot_fontbutton_on_set()
{
    int num = m_combo_font.get_active_row_number();

    font_values[ num ] = m_fontbutton.get_font_name();

    redraw_view();
}


//
// 色選択ダイアログで OK が押された
//
void FontColorPref::slot_colorbutton_on_set()
{
    int num = m_combo_color.get_active_row_number();

    color_values[ num ] = m_colorbutton.get_color();

    redraw_view();
}


//
// プレビューを作成
//
void FontColorPref::preview_create( int mode )
{
#ifdef _DEBUG
    std::cout << "FontColorPref::preview_create : " << "url = " << get_url() << std::endl;
#endif

    struct timeval tv;
    struct timezone tz;
    gettimeofday( &tv, &tz );
    std::string time = MISC::timettostr( tv.tv_sec );

    std::stringstream ss;

    std::string name, mail, id, res;
    switch( mode )
    {
        // dbtree/nodetreebase.cpp に"ID:[#!?]"を判定して、色を変える処理を入れる予定
        case RES_PLURAL:
            name = "名前 </b>◆trip<b>";
            mail = "sage";
            id   = " ID:### (2)";
            res  = "被参照数が1回以上のレスです。";
            break;
        case RES_MANY:
            name = "</b>pref501.isp.ne.jp<b>";
            mail = "";
            id   = " ID:!!! (4)";
            res  = "被参照数が3回以上のレスです。";
            break;
        default:
            name = "名無しさん";
            mail = "sage";
            id   = " ID:???";
            res  = "普通のレスです。";
            break;
    }

    ss << name << "<>";
    ss << mail << "<>";
    ss << time << id << "<>";
    ss << res << "<>\n";

#ifdef _DEBUG
    std::cout << ss.str() << std::endl;
#endif

    m_preview->set_command( "append_dat", ss.str() );
}


//
// プレビューを再描画
//
void FontColorPref::redraw_view()
{
    if( m_preview ) m_preview->redraw_view();
}

