// ライセンス: GPL2

#ifndef _FONTCOLORPREF_H
#define _FONTCOLORPREF_H

#include "skeleton/prefdiag.h"
#include "skeleton/view.h"
#include "config/globalconf.h"
#include "colorid.h"


namespace CORE
{
    // フォント設定用列挙
    enum FONTNAME
    {
        FONTNAME_MAIN = 0,   // スレッドビューなどの基本の物
        FONTNAME_TREE,       // ツリービュー
        FONTNAME_TREE_BOARD, // 現状はツリービューと同じ？
        FONTNAME_POPUP,      // ポップアップ
        FONTNAME_MESSAGE,    // 書き込みウィンドウ

        FONTNAMES
    };

    // プレビューのモード
    enum RES
    {
        RES_DEFAULT,
        RES_PLURAL,
        RES_MANY
    };


    class FontColorPref : public SKELETON::PrefDiag
    {
        // フォントの設定
        std::vector< std::string > font_names;
        std::vector< std::string > font_values;
        bool strict_char_width;

        Gtk::ComboBoxText m_combo_font;
        Gtk::FontButton m_fontbutton;
        Gtk::CheckButton m_checkbutton_font;
        Gtk::HBox m_hbox_font;
        Gtk::VBox m_vbox_font;
        Gtk::Frame m_frame_font;

        void set_font_settings( const std::string& name, const std::string& value );
        void slot_combo_font_changed();
        void slot_fontbutton_on_set();
        void slot_checkbutton_font_toggled();

        // 色の設定
        std::vector< std::string > color_names;
        std::vector< Gdk::Color > color_values;

        Gtk::ComboBoxText m_combo_color;
        Gtk::ColorButton m_colorbutton;
        Gtk::HBox m_hbox_color;
        Gtk::VBox m_vbox_color;
        Gtk::Frame m_frame_color;

        void set_color_settings( const std::string& name, const std::string& value );
        void slot_combo_color_changed();
        void slot_colorbutton_on_set();

        // プレビュー
        Gtk::Frame m_frame_preview;
        SKELETON::View* m_preview;

        void preview_create( int mode = 0 );
        void redraw_view();

        // ウィジェットを追加
        void pack_widget();

        // OK が押された
        virtual void slot_ok_clicked();

      public:

        FontColorPref( const std::string& url );
        ~FontColorPref();
    };
}

#endif
